@@ -5,13 +5,14 @@
 // Exhaustive testing of regular expression matching.
 
 #include <stddef.h>
-#include <memory>
+
 #include <string>
 #include <vector>
 
 #include "gtest/gtest.h"
-#include "util/utf.h"
 #include "re2/testing/exhaustive_tester.h"
+#include "re2/testing/regexp_generator.h"
+#include "util/utf.h"
 
 namespace re2 {
 
@@ -97,4 +98,3 @@ TEST(InterestingUTF8, AB) {
 }
 
 }  // namespace re2
-
