@@ -20,17 +20,31 @@
 // Then RunPostfix turns each sequence into a regular expression
 // and passes the regexp to HandleRegexp.
 
-#include <string.h>
-#include <string>
-#include <stack>
-#include <vector>
-#include "util/test.h"
 #include "re2/testing/regexp_generator.h"
 
+#include <stddef.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <string.h>
+
+#include <memory>
+#include <random>
+#include <stack>
+#include <string>
+#include <vector>
+
+#include "absl/base/macros.h"
+#include "absl/log/absl_check.h"
+#include "absl/log/absl_log.h"
+#include "absl/strings/escaping.h"
+#include "absl/strings/str_format.h"
+#include "absl/strings/string_view.h"
+#include "util/utf.h"
+
 namespace re2 {
 
 // Returns a vector of the egrep regexp operators.
-const vector<string>& RegexpGenerator::EgrepOps() {
+const std::vector<std::string>& RegexpGenerator::EgrepOps() {
   static const char *ops[] = {
     "%s%s",
     "%s|%s",
@@ -39,43 +53,40 @@ const vector<string>& RegexpGenerator::EgrepOps() {
     "%s?",
     "%s\\C*",
   };
-  static vector<string> v(ops, ops + arraysize(ops));
+  static std::vector<std::string> v(ops, ops + ABSL_ARRAYSIZE(ops));
   return v;
 }
 
 RegexpGenerator::RegexpGenerator(int maxatoms, int maxops,
-                                 const vector<string>& atoms,
-                                 const vector<string>& ops)
+                                 const std::vector<std::string>& atoms,
+                                 const std::vector<std::string>& ops)
     : maxatoms_(maxatoms), maxops_(maxops), atoms_(atoms), ops_(ops) {
   // Degenerate case.
-  if (atoms_.size() == 0)
+  if (atoms_.empty())
     maxatoms_ = 0;
-  if (ops_.size() == 0)
+  if (ops_.empty())
     maxops_ = 0;
 }
 
 // Generates all possible regular expressions (within the parameters),
 // calling HandleRegexp for each one.
 void RegexpGenerator::Generate() {
-  vector<string> postfix;
+  std::vector<std::string> postfix;
   GeneratePostfix(&postfix, 0, 0, 0);
 }
 
 // Generates random regular expressions, calling HandleRegexp for each one.
-void RegexpGenerator::GenerateRandom(int32 seed, int n) {
-  ACMRandom acm(seed);
-  acm_ = &acm;
+void RegexpGenerator::GenerateRandom(int32_t seed, int n) {
+  rng_.seed(seed);
 
   for (int i = 0; i < n; i++) {
-    vector<string> postfix;
+    std::vector<std::string> postfix;
     GenerateRandomPostfix(&postfix, 0, 0, 0);
   }
-
-  acm_ = NULL;
 }
 
 // Counts and returns the number of occurrences of "%s" in s.
-static int CountArgs(const string& s) {
+static int CountArgs(const std::string& s) {
   const char *p = s.c_str();
   int n = 0;
   while ((p = strstr(p, "%s")) != NULL) {
@@ -98,8 +109,8 @@ static int CountArgs(const string& s) {
 //
 // The initial call should be GeneratePostfix([empty vector], 0, 0, 0).
 //
-void RegexpGenerator::GeneratePostfix(vector<string>* post, int nstk,
-                                      int ops, int atoms) {
+void RegexpGenerator::GeneratePostfix(std::vector<std::string>* post,
+                                      int nstk, int ops, int atoms) {
   if (nstk == 1)
     RunPostfix(*post);
 
@@ -121,7 +132,7 @@ void RegexpGenerator::GeneratePostfix(vector<string>* post, int nstk,
   // Add operators if there are enough arguments.
   if (ops < maxops_) {
     for (size_t i = 0; i < ops_.size(); i++) {
-      const string& fmt = ops_[i];
+      const std::string& fmt = ops_[i];
       int nargs = CountArgs(fmt);
       if (nargs <= nstk) {
         post->push_back(fmt);
@@ -134,11 +145,18 @@ void RegexpGenerator::GeneratePostfix(vector<string>* post, int nstk,
 
 // Generates a random postfix command sequence.
 // Stops and returns true once a single sequence has been generated.
-bool RegexpGenerator::GenerateRandomPostfix(vector<string>* post, int nstk,
-                                            int ops, int atoms) {
+bool RegexpGenerator::GenerateRandomPostfix(std::vector<std::string>* post,
+                                            int nstk, int ops, int atoms) {
+  std::uniform_int_distribution<int> random_stop(0, maxatoms_ - atoms);
+  std::uniform_int_distribution<int> random_bit(0, 1);
+  std::uniform_int_distribution<int> random_ops_index(
+      0, static_cast<int>(ops_.size()) - 1);
+  std::uniform_int_distribution<int> random_atoms_index(
+      0, static_cast<int>(atoms_.size()) - 1);
+
   for (;;) {
     // Stop if we get to a single element, but only sometimes.
-    if (nstk == 1 && acm_->Uniform(maxatoms_ + 1 - atoms) == 0) {
+    if (nstk == 1 && random_stop(rng_) == 0) {
       RunPostfix(*post);
       return true;
     }
@@ -150,8 +168,8 @@ bool RegexpGenerator::GenerateRandomPostfix(vector<string>* post, int nstk,
       return false;
 
     // Add operators if there are enough arguments.
-    if (ops < maxops_ && acm_->Uniform(2) == 0) {
-      const string& fmt = ops_[acm_->Uniform(static_cast<int32>(ops_.size()))];
+    if (ops < maxops_ && random_bit(rng_) == 0) {
+      const std::string& fmt = ops_[random_ops_index(rng_)];
       int nargs = CountArgs(fmt);
       if (nargs <= nstk) {
         post->push_back(fmt);
@@ -164,8 +182,8 @@ bool RegexpGenerator::GenerateRandomPostfix(vector<string>* post, int nstk,
     }
 
     // Add atoms if there is room.
-    if (atoms < maxatoms_ && acm_->Uniform(2) == 0) {
-      post->push_back(atoms_[acm_->Uniform(static_cast<int32>(atoms_.size()))]);
+    if (atoms < maxatoms_ && random_bit(rng_) == 0) {
+      post->push_back(atoms_[random_atoms_index(rng_)]);
       bool ret = GenerateRandomPostfix(post, nstk + 1, ops, atoms + 1);
       post->pop_back();
       if (ret)
@@ -177,29 +195,31 @@ bool RegexpGenerator::GenerateRandomPostfix(vector<string>* post, int nstk,
 // Interprets the postfix command sequence to create a regular expression
 // passed to HandleRegexp.  The results of operators like %s|%s are wrapped
 // in (?: ) to avoid needing to maintain a precedence table.
-void RegexpGenerator::RunPostfix(const vector<string>& post) {
-  stack<string> regexps;
+void RegexpGenerator::RunPostfix(const std::vector<std::string>& post) {
+  std::stack<std::string> regexps;
   for (size_t i = 0; i < post.size(); i++) {
     switch (CountArgs(post[i])) {
       default:
-        LOG(FATAL) << "Bad operator: " << post[i];
+        ABSL_LOG(FATAL) << "Bad operator: " << post[i];
       case 0:
         regexps.push(post[i]);
         break;
       case 1: {
-        string a = regexps.top();
+        auto fmt = absl::ParsedFormat<'s'>::New(post[i]);
+        ABSL_CHECK(fmt != nullptr);
+        std::string a = regexps.top();
         regexps.pop();
-        regexps.push("(?:" + StringPrintf(post[i].c_str(), a.c_str()) + ")");
+        regexps.push("(?:" + absl::StrFormat(*fmt, a) + ")");
         break;
       }
       case 2: {
-        string b = regexps.top();
+        auto fmt = absl::ParsedFormat<'s', 's'>::New(post[i]);
+        ABSL_CHECK(fmt != nullptr);
+        std::string b = regexps.top();
         regexps.pop();
-        string a = regexps.top();
+        std::string a = regexps.top();
         regexps.pop();
-        regexps.push("(?:" +
-                     StringPrintf(post[i].c_str(), a.c_str(), b.c_str()) +
-                     ")");
+        regexps.push("(?:" + absl::StrFormat(*fmt, a, b) + ")");
         break;
       }
     }
@@ -207,16 +227,16 @@ void RegexpGenerator::RunPostfix(const vector<string>& post) {
 
   if (regexps.size() != 1) {
     // Internal error - should never happen.
-    printf("Bad regexp program:\n");
+    absl::PrintF("Bad regexp program:\n");
     for (size_t i = 0; i < post.size(); i++) {
-      printf("  %s\n", CEscape(post[i]).c_str());
+      absl::PrintF("  %s\n", absl::CEscape(post[i]));
     }
-    printf("Stack after running program:\n");
+    absl::PrintF("Stack after running program:\n");
     while (!regexps.empty()) {
-      printf("  %s\n", CEscape(regexps.top()).c_str());
+      absl::PrintF("  %s\n", absl::CEscape(regexps.top()));
       regexps.pop();
     }
-    LOG(FATAL) << "Bad regexp program.";
+    ABSL_LOG(FATAL) << "Bad regexp program.";
   }
 
   HandleRegexp(regexps.top());
@@ -226,14 +246,14 @@ void RegexpGenerator::RunPostfix(const vector<string>& post) {
 }
 
 // Split s into an vector of strings, one for each UTF-8 character.
-vector<string> Explode(const StringPiece& s) {
-  vector<string> v;
+std::vector<std::string> Explode(absl::string_view s) {
+  std::vector<std::string> v;
 
-  for (const char *q = s.begin(); q < s.end(); ) {
+  for (const char *q = s.data(); q < s.data() + s.size(); ) {
     const char* p = q;
     Rune r;
     q += chartorune(&r, q);
-    v.push_back(string(p, q - p));
+    v.push_back(std::string(p, q - p));
   }
 
   return v;
@@ -241,23 +261,23 @@ vector<string> Explode(const StringPiece& s) {
 
 // Split string everywhere a substring is found, returning
 // vector of pieces.
-vector<string> Split(const StringPiece& sep, const StringPiece& s) {
-  vector<string> v;
+std::vector<std::string> Split(absl::string_view sep, absl::string_view s) {
+  std::vector<std::string> v;
 
-  if (sep.size() == 0)
+  if (sep.empty())
     return Explode(s);
 
-  const char *p = s.begin();
-  for (const char *q = s.begin(); q + sep.size() <= s.end(); q++) {
-    if (StringPiece(q, sep.size()) == sep) {
-      v.push_back(string(p, q - p));
+  const char *p = s.data();
+  for (const char *q = s.data(); q + sep.size() <= s.data() + s.size(); q++) {
+    if (absl::string_view(q, sep.size()) == sep) {
+      v.push_back(std::string(p, q - p));
       p = q + sep.size();
       q = p - 1;  // -1 for ++ in loop
       continue;
     }
   }
-  if (p < s.end())
-    v.push_back(string(p, s.end() - p));
+  if (p < s.data() + s.size())
+    v.push_back(std::string(p, s.data() + s.size() - p));
   return v;
 }
 
